(*
	Single-Instance Utility Demo

	Copyright (c) 2016, Sergey A Kryukov
	http://www.SAKryukov.org
*)
unit fMain;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, ComCtrls, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
   ExtCtrls, SingleInstanceUtility;

type
TFormMain = class(TForm)
   Instructions: TMemo;
   PanelPad: TPanel;
   PageControl: TPageControl;
strict private
   SingleInstanceUtility: TSingleInstanceUtility;
   procedure AcceptCommandLineFromSecondInstance(commandLine: string);
public
   constructor Create(TheOwner: TComponent); override;
   destructor Destroy; override;
end {TFormMain};

implementation
uses LCLType;

{$R *.lfm}

procedure TFormMain.AcceptCommandLineFromSecondInstance(commandLine: string);
var
   fmt: string = 'File: "%s".';
   page: TTabSheet;
   memo: TMemo;
   fileName: string;
   files: array of string;
begin
   Application.Minimize; // this is what really...
   Application.Restore;  // brings a form to top
   Application.BringToFront; // just in case, for orher platforms
   memo := nil;
   files := TSingleInstanceUtility.DeserializeCommandLine(commandLine);
   for fileName in files do begin
     if FileExists(fileName) then begin
        if not Assigned(PageControl) then begin
           PageControl := TPageControl.Create(Application);
           PageControl.Align := alClient;
           PageControl.Parent := self;
           Width := Width * 2;
           self.PanelPad.Hide;
        end {if};
        page := TTabSheet.Create(PageControl);
        page.Caption := IntToStr(PageControl.PageCount + 1);
        memo := TMemo.Create(Application);
        memo.Align := alClient;
        memo.Lines.LoadFromFile(fileName);
        memo.Parent := page;
        memo.ScrollBars := ssAutoVertical;
        page.PageControl := PageControl;
        PageControl.ActivePage := page;
     end else
        Application.MessageBox(
           pChar(Format(fmt, [fileName])),
           'File from the second instance',
           MB_ICONINFORMATION);
   end {loop};
   if Assigned(memo) then memo.SetFocus;
end {TFormMain.AcceptCommandLineFromSecondInstance};

constructor TFormMain.Create(TheOwner: TComponent);
begin
   inherited Create(TheOwner);
   SingleInstanceUtility := TSingleInstanceUtility.Create;
   SingleInstanceUtility.OnCommandLineFromSecondInstance := @AcceptCommandLineFromSecondInstance;
   Instructions.Text := 'Now start the second instance of the same application '
   + 'with some command line parameters.' + LineEnding + LineEnding
   + 'In standard interpretation, the parameters should be file names passed '
   + 'to the first instance of the application, '
   + 'but actually they can be any strings.';
end {TFormMain.Create};

destructor TFormMain.Destroy;
begin
   SingleInstanceUtility.Free;
   inherited Destroy;
end {TFormMain.Destroy};

end.

