{<pre><i>
  Copyright (C) 1999-2016, Sergey A Kryukov
  <p><a href="http://www.SAKryukov.org">http://www.SAKryukov.org</a></p>
  Description
  Command Line utility with .NET style options
      option format:
         -key           // default, can also be interpreted as "enable"
         -key+          // enable
         -key-          // disable
         -key:value     // value
      First "-" (option delimiter) can be "/"
      Keys are not case-sensitive, value are
      non-option parameter:
         filename       // typically filename, can be anything without first "-", "/"
         It is recognized as something without "-" or "/" delimiter
         and can be freely interlaced with options
</i>}
unit CmdLinePlus;
interface
uses Classes, SysUtils;

//<strong>
type
tCommandLineOptionStatus = (closUndefined, closDefault, closPlus, closMinus, closValue);
//                          not mentioned  -OPT         -OPT+     -OPT-      -OPT:value
tCommandLineOption = record
   Status: tCommandLineOptionStatus;
   Key, Value: string;
end {tCommandLineOption};
const closPresent = [closDefault, closPlus]; // -OPT+ or -OPT

type TCommandLine = class
private
   FFileCount: integer;
   FFileNames: array of string;
   FOptionCount: integer;
   FOptions: array of tCommandLineOption;
   FExeName, FExeNamePure, FExePath: string;
   class function EmptyOption: tCommandLineOption;
private
   procedure Scan(externalSource: array of string);
   function GetFilename(Index: integer): string;
   function GetOptionByIndex(Index: integer): tCommandLineOption;
   function GetOptionByKey(Key: string): tCommandLineOption;
public
   constructor Create;
   constructor Create(externalSource: array of string);
   property ExeName: string read FExeName;
   property ExeNamePure: string read FExeNamePure;
   property ExePath: string read FExePath;
   property FileCount: integer read FFileCount;
   property Filename[Index: integer]: string read GetFilename;
   property OptionCount: integer read FOptionCount;
   property Option[Index: integer]: tCommandLineOption read GetOptionByIndex;
   property OptionByKey[Key: string]: tCommandLineOption read GetOptionByKey;
end {TCommandLine};
//</strong>

var
   CommandLine: TCommandLine = nil;

const
   OPTION_CHAR1 = '-';
   OPTION_CHAR2 = '/';
   OPTION_SET = [OPTION_CHAR1, OPTION_CHAR2];
   OPTION_PLUS = '+';
   OPTION_MINUS = '-';
   OPTION_SWITCH_SET = [OPTION_PLUS, OPTION_MINUS];
   OPTION_VALUE_DELIM = ':';

implementation
//--============================================================================

constructor TCommandLine.Create;
begin
   inherited Create;
   Scan([]);
end {TCommandLine.Create};
constructor TCommandLine.Create(externalSource: array of string);
begin
   inherited Create;
   Scan(externalSource);
end {TCommandLine.Create};

function TCommandLine.GetFilename(Index: integer): string;
begin
   Result := FFilenames[Index];
end {TCommandLine.GetFilename};

function TCommandLine.GetOptionByIndex(Index: integer): tCommandLineOption;
begin
   if Index>FOptionCount-1 then begin
      Result := EmptyOption; exit;
   end {if};
   Result := FOptions[Index];
end {TCommandLine.GetOptionByIndex};

function TCommandLine.GetOptionByKey(Key: string): tCommandLineOption;
var
   jj, cnt: integer;
   opt: tCommandLineOption;
begin // TCommandLine.GetOptionByKey
   cnt := FOptionCount;
   for jj := 0 to cnt-1 do begin
      opt := GetOptionByIndex(jj);
      if opt.Key=UpperCase(key) then begin
         Result := opt; exit;
      end {if};
   end {loop};
   Result := EmptyOption;
end {TCommandLine.GetOptionByKey};

class function TCommandLine.EmptyOption: tCommandLineOption;
begin // TCommandLine.EmptyOption
   Result.Status := closUndefined;
   FillChar(Result, sizeof(Result), 0);
end {TCommandLine.EmptyOption};

procedure TCommandLine.Scan(externalSource: array of string);
   function ExtractFNamePure(const FName: string): string;
   var
      ext: string;
   begin // OPTION_VALUE_DELIM
      Result := ExtractFileName(FName);
      ext := ExtractFileExt(Result);
      SetLength(Result, Length(Result)-Length(ext));
   end {ExtractFNamePure};
   function ExractOptionStatus(var opt: tCommandLineOption): boolean; // false if error
   var
      value: string;
      vallen: integer;
   begin // ExractOptionStatus
      Result := true;
      opt.Status := closValue;
      value := opt.Value;
      vallen := Length(value);
      if vallen<1 then begin
         opt.Status := closDefault; // so far
         value := opt.Key;
         vallen := Length(value);
      end {if};
      if vallen<1 then begin Result := false; exit; end;
      case value[vallen] of
         OPTION_PLUS:
            if opt.Status=closValue then begin
               Result := false; exit;
            end else begin
               opt.Status := closPlus;
            end {if};
         OPTION_MINUS:
            if opt.Status=closValue then begin
               Result := false; exit;
            end else begin
               opt.Status := closMinus;
            end {if};
      end {case};
   end {ExractOptionStatus};
   function FixKey(const key: string): string;
   var
      len: integer;
   begin // FixKey
      Result := key;
      len := Length(Result);
      if len<1 then exit;
      if Result[len] in OPTION_SWITCH_SET then
         SetLength(Result, len-1);
   end {FixKey};
   function ParseAsOption(const src: string; out opt: tCommandLineOption; out error: boolean): boolean;
   var
      pair: string;
      srclen, colonpos: integer;
   begin // ParseAsOption
      error := false;
      Result := src[1] //SA!!! always there
         in OPTION_SET;
      if not Result then exit;
      srclen := Length(src);
      pair := system.Copy(src, 2, srclen);
      if Length(pair)<1 then begin error := true; exit; end;
      colonpos := system.pos(OPTION_VALUE_DELIM, pair);
      if colonpos>0 then begin
         opt.Key := UpperCase(system.Copy(pair, 1, colonpos-1));
         opt.Value := system.Copy(pair, colonpos+1, srclen);
         if (Length(opt.Key)<1) then begin error := true; exit; end;
      end else begin
         opt.Key := UpperCase(pair);
         opt.Value := '';
      end {if};
      error := not ExractOptionStatus(opt);
      opt.Key := FixKey(opt.Key);
   end {ParseAsOption};
var
   index, len, iterationCount: integer;
   ss: string;
   opt: tCommandLineOption; optionerror, useExternalSource: boolean;
begin // TCommandLine.Scan
   FExeName := ParamStr(0);
   FExeNamePure := ExtractFNamePure(ExtractFileName(FExeName));
   FExePath := ExtractFilePath(FExeName);
   useExternalSource := Length(externalSource) > 1;
   if not useExternalSource then iterationCount := ParamCount else iterationCount := Length(externalSource);
   for index := 1 to iterationCount do begin
      if not useExternalSource then ss := ParamStr(index) else ss := externalSource[index - 1];
      if ParseAsOption(ss, opt, optionerror) then begin
         if optionerror then continue;
         len := Length(FOptions);
         SetLength(FOptions, len+1);
         FOptions[len] := opt;
      end else begin
         len := Length(FFileNames);
         SetLength(FFileNames, len+1);
         FFileNames[len] := ss;
      end {if};
   end {loop};
   FOptionCount := Length(FOptions);
   FFileCount := Length(FFileNames);
end {TCommandLine.Scan};

initialization
   CommandLine := TCommandLine.Create;
finalization
   CommandLine.Free;
end.//<pre>
